import type { Matchers } from "./matchers.js";
interface Expected {
    toBe(candidate: any): void;
    toEqual(candidate: any): void;
    toBeTruthy(): void;
    toBeFalsy(): void;
    toBeDefined(): void;
    toBeInstanceOf(clazz: any): void;
    toBeUndefined(): void;
    toBeNull(): void;
    toBeNaN(): void;
    toMatch(pattern: RegExp | string): void;
    toHaveProperty(propName: string): void;
    toHaveLength(length: number): void;
    toContain(item: any): void;
    toThrow(error?: RegExp | string): void;
    toBeGreaterThan(number: number): void;
    toBeGreaterThanOrEqual(number: number): void;
    toBeLessThan(number: number): void;
    toBeLessThanOrEqual(number: number): void;
    toHaveBeenCalled(): void;
    toHaveBeenCalledTimes(number: number): void;
    toHaveBeenCalledWith(...args: any[]): void;
    toHaveBeenLastCalledWith(...args: any[]): void;
    toHaveBeenNthCalledWith(nthCall: number, ...args: any[]): void;
    toHaveReturned(): void;
    toHaveReturnedTimes(number: number): void;
    toHaveReturnedWith(value: any): void;
    toHaveLastReturnedWith(value: any): void;
    toHaveNthReturnedWith(nthCall: number, value: any): void;
    not: Expected;
    resolves: Expected;
    rejects: Expected;
}
export declare function expect(value: any): Expected;
export declare function addMatchers(newMatchers: Matchers): void;
export {};
